@@ -1,127 +1,110 @@
-import os
 import hashlib
+import os
 import traceback
 from configparser import ConfigParser, NoSectionError
 
 from fsbc.paths import Paths
-
-from fsgs.context import fsgs
-from fsgs.ChecksumTool import ChecksumTool
-from fsgs.amiga.Amiga import Amiga
-from fsgs.amiga.ValueConfigLoader import ValueConfigLoader
+from fsbc.signal import Signal
+from fsgs.checksumtool import ChecksumTool
 from fsgs.FSGSDirectories import FSGSDirectories
+from fsgs.amiga.amiga import Amiga
+from fsgs.amiga.valueconfigloader import ValueConfigLoader
+from fsgs.context import fsgs
+from fsgs.platform import PlatformHandler
 from launcher.option import Option
 from .launcher_settings import LauncherSettings
-from fsbc.signal import Signal
-
-
-# the order of the following keys is significant (for some keys).
-# multiple options should be set in this order since some options will
-# implicitly change other options. examples:
-# - amiga model can change kickstart and joystick modes
-# - file options should be set before corresponding sha1 options
-from fsgs.platform import PlatformHandler
 
 cfg = [
-    ("amiga_model",           "",     "checksum", "sync"),
-    ("ntsc_mode",             "",         "checksum", "sync"),
-    ("accuracy",              "",         "checksum", "sync"),
-    ("chip_memory",           "",         "checksum", "sync"),
-    ("slow_memory",           "",         "checksum", "sync"),
-    ("fast_memory",           "",         "checksum", "sync"),
-    ("zorro_iii_memory",      "",         "checksum", "sync"),
-    ("bsdsocket_library",     "",         "checksum", "sync"),
-    ("uaegfx_card",           "",         "checksum", "sync"),
-    ("joystick_port_0",       ""),
-    ("joystick_port_0_mode",  "",         "checksum", "sync"),
-    ("joystick_port_0_autofire",  "",     "checksum", "sync"),
-    ("joystick_port_1",       ""),
-    ("joystick_port_1_mode",  "",         "checksum", "sync"),
-    ("joystick_port_1_autofire",  "",     "checksum", "sync"),
-    ("joystick_port_2",       ""),
-    ("joystick_port_2_mode",  "",         "checksum", "sync"),
-    ("joystick_port_2_autofire",  "",     "checksum", "sync"),
-    ("joystick_port_3",       ""),
-    ("joystick_port_3_mode",  "",         "checksum", "sync"),
-    ("joystick_port_3_autofire",  "",     "checksum", "sync"),
-
-    ("floppy_drive_count",    "",         "checksum", "sync"),
-    ("cdrom_drive_count",     "",         "checksum", "sync"),
-
+    ("amiga_model", "", "checksum", "sync"),
+    ("ntsc_mode", "", "checksum", "sync"),
+    ("accuracy", "", "checksum", "sync"),
+    ("chip_memory", "", "checksum", "sync"),
+    ("slow_memory", "", "checksum", "sync"),
+    ("fast_memory", "", "checksum", "sync"),
+    ("zorro_iii_memory", "", "checksum", "sync"),
+    ("bsdsocket_library", "", "checksum", "sync"),
+    ("uaegfx_card", "", "checksum", "sync"),
+    ("joystick_port_0", ""),
+    ("joystick_port_0_mode", "", "checksum", "sync"),
+    ("joystick_port_0_autofire", "", "checksum", "sync"),
+    ("joystick_port_1", ""),
+    ("joystick_port_1_mode", "", "checksum", "sync"),
+    ("joystick_port_1_autofire", "", "checksum", "sync"),
+    ("joystick_port_2", ""),
+    ("joystick_port_2_mode", "", "checksum", "sync"),
+    ("joystick_port_2_autofire", "", "checksum", "sync"),
+    ("joystick_port_3", ""),
+    ("joystick_port_3_mode", "", "checksum", "sync"),
+    ("joystick_port_3_autofire", "", "checksum", "sync"),
+    ("floppy_drive_count", "", "checksum", "sync"),
+    ("cdrom_drive_count", "", "checksum", "sync"),
     # this is not an Amiga device, so no need to checksum / sync
-    ("joystick_port_4_mode",  "",         "custom"),
-
-    ("kickstart_file",        ""),
-    ("x_kickstart_file",      "",                             "no_save"),
-    ("x_kickstart_file_sha1", "",         "checksum", "sync", "no_save"),
-    ("kickstart_ext_file",    ""),
-    ("x_kickstart_ext_file",  "",                             "no_save"),
-    ("x_kickstart_ext_file_sha1", "",     "checksum", "sync", "no_save"),
-
-    ("x_whdload_args",        "",         "checksum", "sync"),
-    ("x_whdload_version",     "",     "checksum", "sync"),
-    ("floppy_drive_count",    "",         "checksum", "sync"),
-    ("floppy_drive_speed",    "",         "checksum", "sync"),
-    ("cdrom_drive_count",     "",         "checksum", "sync"),
-    ("dongle_type",           "",         "checksum", "sync"),
-
-    ("__netplay_game",        "",         "checksum", "sync"),
-    ("__netplay_password",    "",         "checksum", "sync"),
-    ("__netplay_players",     "",         "checksum", "sync"),
-    ("__netplay_port",        "",                     "sync"),
-    ("__netplay_addresses",   "",         "checksum", "sync"),
-    ("__netplay_host",        ""),
-    ("__netplay_ready",       ""),
-    ("__netplay_state_dir_name",  "",     "checksum", "sync"),
-    ("__version",             "FIXME"),
-    ("__error",               ""),
-    ("x_game_uuid",           ""),
-    ("x_game_xml_path",       ""),
-    ("title",                 "",                             "custom"),
-    ("sub_title",             "",                             "custom"),
-    ("viewport",              "",                             "custom"),
-
-    ("year",                  ""),
-    ("developer",             ""),
-    ("publisher",             ""),
-    ("languages",             ""),
-    ("players",               ""),
-    ("protection",            ""),
-    ("hol_url",               ""),
-    ("wikipedia_url",         ""),
-    ("database_url",          ""),
-    ("lemon_url",             ""),
-    ("mobygames_url",         ""),
-    ("amigamemo_url",         ""),
-    ("whdload_url",           ""),
-    ("mobygames_url",         ""),
-    ("thelegacy_url",         ""),
-    ("homepage_url",         ""),
-    ("longplay_url",          ""),
-    ("__variant_rating",      ""),
-    ("variant_rating",        ""),
-    ("variant_uuid",          ""),
-    
-    ("download_file",         ""),
-    ("download_page",         ""),
-    ("download_terms",        ""),
-    ("download_notice",       ""),
-
-    ("x_missing_files",       ""),
-    ("x_game_notice",         ""),
-    ("x_variant_notice",      ""),
-    ("x_variant_warning",      ""),
-    ("x_variant_error",      ""),
-    ("x_joy_emu_conflict",    ""),
-
-    ("screen1_sha1",          ""),
-    ("screen2_sha1",          ""),
-    ("screen3_sha1",          ""),
-    ("screen4_sha1",          ""),
-    ("screen5_sha1",          ""),
-    ("front_sha1",            ""),
-    ("title_sha1",            ""),
-
+    ("joystick_port_4_mode", "", "custom"),
+    ("kickstart_file", ""),
+    ("x_kickstart_file", "", "no_save"),
+    ("x_kickstart_file_sha1", "", "checksum", "sync", "no_save"),
+    ("kickstart_ext_file", ""),
+    ("x_kickstart_ext_file", "", "no_save"),
+    ("x_kickstart_ext_file_sha1", "", "checksum", "sync", "no_save"),
+    (Option.X_WHDLOAD_ARGS, "", "checksum", "sync"),
+    (Option.X_WHDLOAD_VERSION, "", "checksum", "sync"),
+    # (Option.WHDLOAD_SPLASH_DELAY, "", "checksum", "sync"),
+    # (Option.WHDLOAD_QUIT_KEY, "", "checksum", "sync"),
+    ("floppy_drive_count", "", "checksum", "sync"),
+    ("floppy_drive_speed", "", "checksum", "sync"),
+    ("cdrom_drive_count", "", "checksum", "sync"),
+    ("dongle_type", "", "checksum", "sync"),
+    ("__netplay_game", "", "checksum", "sync"),
+    ("__netplay_password", "", "checksum", "sync"),
+    ("__netplay_players", "", "checksum", "sync"),
+    ("__netplay_port", "", "sync"),
+    ("__netplay_addresses", "", "checksum", "sync"),
+    ("__netplay_host", ""),
+    ("__netplay_ready", ""),
+    ("__netplay_state_dir_name", "", "checksum", "sync"),
+    ("__version", "FIXME"),
+    ("__error", ""),
+    ("x_game_uuid", ""),
+    ("x_game_xml_path", ""),
+    ("title", "", "custom"),
+    ("sub_title", "", "custom"),
+    ("viewport", "", "custom"),
+    ("year", ""),
+    ("developer", ""),
+    ("publisher", ""),
+    ("languages", ""),
+    ("players", ""),
+    ("protection", ""),
+    ("hol_url", ""),
+    ("wikipedia_url", ""),
+    ("database_url", ""),
+    ("lemon_url", ""),
+    ("mobygames_url", ""),
+    ("amigamemo_url", ""),
+    ("whdload_url", ""),
+    ("mobygames_url", ""),
+    ("thelegacy_url", ""),
+    ("homepage_url", ""),
+    ("longplay_url", ""),
+    ("variant_uuid", ""),
+    ("game_uuid", ""),
+    ("download_file", ""),
+    ("download_page", ""),
+    ("download_terms", ""),
+    ("download_notice", ""),
+    ("x_missing_files", ""),
+    ("x_game_notice", ""),
+    ("x_variant_notice", ""),
+    ("x_variant_warning", ""),
+    ("x_variant_error", ""),
+    ("x_joy_emu_conflict", ""),
+    ("screen1_sha1", ""),
+    ("screen2_sha1", ""),
+    ("screen3_sha1", ""),
+    ("screen4_sha1", ""),
+    ("screen5_sha1", ""),
+    ("front_sha1", ""),
+    ("title_sha1", ""),
     ("mouse_integration", "", "checksum", "sync"),
     ("cdrom_drive_0_delay", "", "checksum", "sync"),
     ("cpu", "", "checksum", "sync"),
@@ -135,40 +118,99 @@ cfg = [
     ("jit_compiler", "", "checksum", "sync"),
     ("__database", ""),
     ("platform", ""),
-    ("floppy_drive_volume_empty", "", "sync"),
     ("save_disk", "", "checksum", "sync"),
     ("network_card", "", "checksum", "sync"),
     ("freezer_cartridge", "", "checksum", "sync"),
+    (Option.A7800_A78_HEADER, "", "checksum", "sync"),
+    (Option.A7800_MODEL, "", "checksum", "sync"),
+    (Option.A7800_PORT_1_TYPE, "", "checksum", "sync"),
+    (Option.A7800_PORT_2_TYPE, "", "checksum", "sync"),
+    (Option.NES_MODEL, "", "checksum", "sync"),
+    (Option.ATARI_MODEL, "", "checksum", "sync"),
+    (Option.C64_MODEL, "", "checksum", "sync"),
+    (Option.CARTRIDGE_SLOT, "", "checksum", "sync"),
+    (Option.FILE_LIST, "", "custom", "sync"),
+    (Option.FLOPPY_DRIVE_VOLUME_EMPTY, "", "sync"),
+    (Option.GAME_NAME, ""),
+    ("mame_rom_set", "", "custom", "checksum", "sync"),
+    (Option.NEOGEO_MODEL, "", "checksum", "sync"),
+    (Option.NES_INES_HEADER, "", "checksum", "sync"),
+    (Option.NES_MODEL, "", "checksum", "sync"),
+    (Option.SMD_EMULATOR, "", "checksum", "sync"),
+    (Option.SMD_MODEL, "", "checksum", "sync"),
+    (Option.SMD_PORT_1_TYPE, "", "checksum", "sync"),
+    (Option.SMD_PORT_2_TYPE, "", "checksum", "sync"),
+    ("refresh_rate", "", "custom", "checksum", "sync"),
+    (Option.TAPE_DRIVE_0, "", "checksum", "sync"),
+    ("tape_image_0", "", "checksum", "sync"),
+    (Option.VARIANT_NAME, ""),
+    (Option.ZXS_MODEL, "", "checksum", "sync"),
 ]
 
 for _i in range(Amiga.MAX_FLOPPY_DRIVES):
     cfg.append(("floppy_drive_{0}".format(_i), ""))
-    cfg.append(("x_floppy_drive_{0}_sha1".format(_i),
-                "", "checksum", "sync", "no_save"))
+    cfg.append(
+        (
+            "x_floppy_drive_{0}_sha1".format(_i),
+            "",
+            "checksum",
+            "sync",
+            "no_save",
+        )
+    )
 for _i in range(Amiga.MAX_FLOPPY_IMAGES):
     cfg.append(("floppy_image_{0}".format(_i), ""))
-    cfg.append(("x_floppy_image_{0}_sha1".format(_i),
-                "", "checksum", "sync", "no_save"))
+    cfg.append(
+        (
+            "x_floppy_image_{0}_sha1".format(_i),
+            "",
+            "checksum",
+            "sync",
+            "no_save",
+        )
+    )
 for _i in range(Amiga.MAX_CDROM_DRIVES):
     cfg.append(("cdrom_drive_{0}".format(_i), ""))
-    cfg.append(("x_cdrom_drive_{0}_sha1".format(_i),
-                "", "checksum", "sync", "no_save"))
+    cfg.append(
+        (
+            "x_cdrom_drive_{0}_sha1".format(_i),
+            "",
+            "checksum",
+            "sync",
+            "no_save",
+        )
+    )
 for _i in range(Amiga.MAX_CDROM_IMAGES):
     cfg.append(("cdrom_image_{0}".format(_i), ""))
-    cfg.append(("x_cdrom_image_{0}_sha1".format(_i),
-                "", "checksum", "sync", "no_save"))
+    cfg.append(
+        (
+            "x_cdrom_image_{0}_sha1".format(_i),
+            "",
+            "checksum",
+            "sync",
+            "no_save",
+        )
+    )
 for _i in range(Amiga.MAX_HARD_DRIVES):
     cfg.append(("hard_drive_{0}".format(_i), ""))
-    cfg.append(("hard_drive_{0}_label".format(_i),
-                "", "checksum", "sync", "custom"))
-    cfg.append(("hard_drive_{0}_priority".format(_i),
-                "", "checksum", "sync", "custom"))
-    cfg.append(("x_hard_drive_{0}_sha1".format(_i),
-                "", "checksum", "sync", "no_save"))
+    cfg.append(
+        ("hard_drive_{0}_label".format(_i), "", "checksum", "sync", "custom")
+    )
+    cfg.append(
+        (
+            "hard_drive_{0}_priority".format(_i),
+            "",
+            "checksum",
+            "sync",
+            "custom",
+        )
+    )
+    cfg.append(
+        ("x_hard_drive_{0}_sha1".format(_i), "", "checksum", "sync", "no_save")
+    )
 
 
 class LauncherConfig(object):
-
     config_keys = [x[0] for x in cfg]
 
     default_config = {}
@@ -192,20 +234,30 @@ class LauncherConfig(object):
 
     reset_values = {}
     for i in range(Amiga.MAX_FLOPPY_DRIVES):
-        reset_values["floppy_drive_{0}".format(i)] = \
-            ("x_floppy_drive_{0}_sha1".format(i), "")
+        reset_values["floppy_drive_{0}".format(i)] = (
+            "x_floppy_drive_{0}_sha1".format(i),
+            "",
+        )
     for i in range(Amiga.MAX_FLOPPY_IMAGES):
-        reset_values["floppy_image_{0}".format(i)] = \
-            ("x_floppy_image_{0}_sha1".format(i), "")
+        reset_values["floppy_image_{0}".format(i)] = (
+            "x_floppy_image_{0}_sha1".format(i),
+            "",
+        )
     for i in range(Amiga.MAX_CDROM_DRIVES):
-        reset_values["cdrom_drive_{0}".format(i)] = \
-            ("x_cdrom_drive_{0}_sha1".format(i), "")
+        reset_values["cdrom_drive_{0}".format(i)] = (
+            "x_cdrom_drive_{0}_sha1".format(i),
+            "",
+        )
     for i in range(Amiga.MAX_CDROM_IMAGES):
-        reset_values["cdrom_image_{0}".format(i)] = \
-            ("x_cdrom_image_{0}_sha1".format(i), "")
+        reset_values["cdrom_image_{0}".format(i)] = (
+            "x_cdrom_image_{0}_sha1".format(i),
+            "",
+        )
     for i in range(Amiga.MAX_HARD_DRIVES):
-        reset_values["hard_drive_{0}".format(i)] = \
-            ("x_hard_drive_{0}_sha1".format(i), "")
+        reset_values["hard_drive_{0}".format(i)] = (
+            "x_hard_drive_{0}_sha1".format(i),
+            "",
+        )
     reset_values["x_kickstart_file"] = ("x_kickstart_file_sha1", "")
     reset_values["x_kickstart_ext_file"] = ("x_kickstart_ext_file_sha1", "")
 
@@ -275,7 +327,8 @@ class LauncherConfig(object):
     def update_kickstart_in_config_dict(cls, config_dict):
         print("update_kickstart_in_config")
         model = config_dict.setdefault(
-            "amiga_model", cls.default_config["amiga_model"])
+            "amiga_model", cls.default_config["amiga_model"]
+        )
 
         kickstart_file = config_dict.setdefault("kickstart_file", "")
         if kickstart_file:
@@ -299,8 +352,9 @@ class LauncherConfig(object):
                 config_dict["x_kickstart_file_sha1"] = Amiga.INTERNAL_ROM_SHA1
 
         if config_dict.setdefault("kickstart_ext_file", ""):
-            config_dict["x_kickstart_ext_file"] = \
-                config_dict["kickstart_ext_file"]
+            config_dict["x_kickstart_ext_file"] = config_dict[
+                "kickstart_ext_file"
+            ]
             # FIXME: set checksum
         else:
             checksums = Amiga.get_model_config(model)["ext_roms"]
@@ -334,10 +388,13 @@ class LauncherConfig(object):
         cls.update_from_config_dict(config_dict)
 
     @classmethod
-    def load_default_config(cls):
-        print("load_default_config")
-        cls.load({})
-        LauncherSettings.set("config_name", "Unnamed Configuration")
+    def load_default_config(cls, platform=""):
+        print("load_default_config platform =", platform)
+        if platform:
+            cls.load({Option.PLATFORM: platform})
+        else:
+            cls.load({})
+        LauncherSettings.set(Option.CONFIG_NAME, "Unnamed Configuration")
         LauncherSettings.set("config_path", "")
         LauncherSettings.set("config_xml_path", "")
 
@@ -380,7 +437,7 @@ class LauncherConfig(object):
             # Mark config as unchanged (i.e. does not need to be saved).
             cls.set("__changed", "0")
 
-        # cls.update_kickstart()
+            # cls.update_kickstart()
 
     @classmethod
     def fix_joystick_ports(cls, config):
@@ -389,7 +446,8 @@ class LauncherConfig(object):
         print("---", config["joystick_port_0"])
         print("---", config["joystick_port_1"])
 
-        from .device_manager import DeviceManager
+        from .devicemanager import DeviceManager
+
         available = DeviceManager.get_joystick_names()
         available.extend(["none", "mouse", "keyboard"])
         available_lower = [x.lower() for x in available]
@@ -483,15 +541,15 @@ class LauncherConfig(object):
 
     @classmethod
     def create_fs_name(cls, name):
-        name = name.replace(':', ' - ')
-        name = name.replace('*', '-')
-        name = name.replace('?', '')
-        name = name.replace('/', '-')
-        name = name.replace('\\', '-')
+        name = name.replace(":", " - ")
+        name = name.replace("*", "-")
+        name = name.replace("?", "")
+        name = name.replace("/", "-")
+        name = name.replace("\\", "-")
         name = name.replace('"', "'")
         for i in range(3):
-            name = name.replace('  ', ' ')
-        while name.endswith('.'):
+            name = name.replace("  ", " ")
+        while name.endswith("."):
             name = name[:-1]
         name = name.strip()
         return name
@@ -543,14 +601,14 @@ class LauncherConfig(object):
         else:
             config_name, ext = os.path.splitext(os.path.basename(path))
 
-        LauncherSettings.set("config_name", config_name)
+        LauncherSettings.set(Option.CONFIG_NAME, config_name)
         LauncherSettings.set("config_xml_path", config_xml_path)
         cls.set("__changed", changed)
         return True
 
     @classmethod
     def load_values(cls, values, uuid=""):
-        print("loading config values", values)
+        # print("loading config values", values)
         platform_id = values.get("platform", "").lower()
 
         if platform_id in ["amiga", "cdtv", "cd32"]:
@@ -558,22 +616,28 @@ class LauncherConfig(object):
             value_config_loader.load_values(values)
             config = value_config_loader.get_config()
             cls.load(config)
-            config_name = config.get("__config_name", "")
-
+            values["__config_name"] = config.get("__config_name")
         else:
             print("Warning: Non-Amiga game loaded")
             platform_handler = PlatformHandler.create(platform_id)
             loader = platform_handler.get_loader(fsgs)
             fsgs.config.load(loader.load_values(values))
+        cls.post_load_values(values)
+
+    @classmethod
+    def post_load_values(cls, values):
+        print("POST_LOAD_VALUES")
+        if values.get("__config_name", ""):
+            print("__config_name was set")
+            config_name = values.get("__config_name", "")
+        else:
             config_name = "{0} ({1})".format(
-                values.get("game_name"), values.get("platform_name"))
-
+                values.get("game_name"), values.get("variant_name")
+            )
         LauncherSettings.set("config_path", "")
-
         if config_name:
             config_name = cls.create_fs_name(config_name)
-
-        LauncherSettings.set("config_name", config_name)
+        LauncherSettings.set(Option.CONFIG_NAME, config_name)
         LauncherSettings.set("config_xml_path", "")
         cls.set("__changed", "0")
 
@@ -650,35 +714,46 @@ class LauncherConfig(object):
             fix_file_checksum(
                 "x_floppy_drive_{0}_sha1".format(i),
                 "floppy_drive_{0}".format(i),
-                FSGSDirectories.get_floppies_dir())
+                FSGSDirectories.get_floppies_dir(),
+            )
         for i in range(Amiga.MAX_FLOPPY_IMAGES):
             fix_file_checksum(
                 "x_floppy_image_{0}_sha1".format(i),
                 "floppy_image_{0}".format(i),
-                FSGSDirectories.get_floppies_dir())
+                FSGSDirectories.get_floppies_dir(),
+            )
         for i in range(Amiga.MAX_CDROM_DRIVES):
             fix_file_checksum(
                 "x_cdrom_drive_{0}_sha1".format(i),
                 "cdrom_drive_{0}".format(i),
-                FSGSDirectories.get_cdroms_dir())
+                FSGSDirectories.get_cdroms_dir(),
+            )
         for i in range(Amiga.MAX_CDROM_IMAGES):
             fix_file_checksum(
                 "x_cdrom_image_{0}_sha1".format(i),
                 "cdrom_image_{0}".format(i),
-                FSGSDirectories.get_cdroms_dir())
+                FSGSDirectories.get_cdroms_dir(),
+            )
         for i in range(Amiga.MAX_HARD_DRIVES):
             fix_file_checksum(
                 "x_hard_drive_{0}_sha1".format(i),
                 "hard_drive_{0}".format(i),
-                FSGSDirectories.get_hard_drives_dir())
+                FSGSDirectories.get_hard_drives_dir(),
+            )
 
         # FIXME: need to handle checksums for Cloanto here
         fix_file_checksum(
-            "x_kickstart_file_sha1", "x_kickstart_file",
-            FSGSDirectories.get_kickstarts_dir(), is_rom=True)
+            "x_kickstart_file_sha1",
+            "x_kickstart_file",
+            FSGSDirectories.get_kickstarts_dir(),
+            is_rom=True,
+        )
         fix_file_checksum(
-            "x_kickstart_ext_file_sha1", "x_kickstart_ext_file",
-            FSGSDirectories.get_kickstarts_dir(), is_rom=True)
+            "x_kickstart_ext_file_sha1",
+            "x_kickstart_ext_file",
+            FSGSDirectories.get_kickstarts_dir(),
+            is_rom=True,
+        )
 
         # Convert uaegfx_card to new graphics_card option
         uae_gfx_card = config.get(Option.UAEGFX_CARD, "")
