@@ -61,6 +61,42 @@ typedef LPEXCEPTION_POINTERS CONTEXT_T;
 #define CONTEXT_RSI(context) (context->ContextRecord->Esi)
 #define CONTEXT_RDI(context) (context->ContextRecord->Edi)
 
+#elif defined(HAVE_STRUCT_UCONTEXT_T_UC_MCONTEXT_GREGS) && defined(CPU_x86_64)
+
+typedef void *CONTEXT_T;
+#define HAVE_CONTEXT_T 1
+#define CONTEXT_RIP(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_RIP])
+#define CONTEXT_RAX(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_RAX])
+#define CONTEXT_RCX(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_RCX])
+#define CONTEXT_RDX(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_RDX])
+#define CONTEXT_RBX(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_RBX])
+#define CONTEXT_RSP(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_RSP])
+#define CONTEXT_RBP(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_RBP])
+#define CONTEXT_RSI(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_RSI])
+#define CONTEXT_RDI(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_RDI])
+#define CONTEXT_R8(context)  (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_R8])
+#define CONTEXT_R9(context)  (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_R9])
+#define CONTEXT_R10(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_R10])
+#define CONTEXT_R11(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_R11])
+#define CONTEXT_R12(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_R12])
+#define CONTEXT_R13(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_R13])
+#define CONTEXT_R14(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_R14])
+#define CONTEXT_R15(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_R15])
+
+#elif defined(HAVE_STRUCT_UCONTEXT_T_UC_MCONTEXT_GREGS) && defined(CPU_i386)
+
+typedef void *CONTEXT_T;
+#define HAVE_CONTEXT_T 1
+#define CONTEXT_RIP(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_EIP])
+#define CONTEXT_RAX(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_EAX])
+#define CONTEXT_RCX(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_ECX])
+#define CONTEXT_RDX(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_EDX])
+#define CONTEXT_RBX(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_EBX])
+#define CONTEXT_RSP(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_ESP])
+#define CONTEXT_RBP(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_EBP])
+#define CONTEXT_RSI(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_ESI])
+#define CONTEXT_RDI(context) (((struct ucontext_t *) context)->uc_mcontext.gregs[REG_EDI])
+
 #elif defined(HAVE_STRUCT_UCONTEXT_UC_MCONTEXT_GREGS) && defined(CPU_x86_64)
 
 typedef void *CONTEXT_T;
@@ -424,6 +460,9 @@ static int handle_access(uintptr_t fault_addr, CONTEXT_T context)
 		(addr >= 0x50000000)) {
 			write_log (_T("JIT: Suspicious address 0x%x in SEGV handler.\n"), addr);
 	}
+	addrbank *ab = &get_mem_bank(addr);
+	if (ab)
+		write_log(_T("JIT: Address bank: %s, address %08x\n"), ab->name ? ab->name : _T("NONE"), addr);
 #endif
 
 	uae_u8* original_target = target;
@@ -576,7 +615,7 @@ static void install_exception_handler(void)
 	sigaction(SIGBUS, &act, NULL);
 #endif
 #else
-	write_log (_T("JIT: No segfault handler installed\n"));
+	write_log (_T("JIT: WARNING: No segfault handler installed\n"));
 #endif
 }
 
