@@ -11,6 +11,7 @@
 #include "autoconf.h"
 #include "options.h"
 #include "blkdev.h"
+#include "clipboard.h"
 #include "custom.h"
 #include "keyboard.h"
 #include "inputdevice.h"
@@ -22,6 +23,7 @@
 #include "uae/fs.h"
 #include "uae/log.h"
 #include "uae/glib.h"
+#include "uae/time.h"
 
 void keyboard_settrans (void);
 libamiga_callbacks g_libamiga_callbacks = {};
@@ -30,7 +32,7 @@ amiga_media_function g_amiga_media_function = NULL;
 
 int g_uae_deterministic_mode = 0;
 int g_amiga_paused = 0;
-
+bool g_fs_uae_jit_compiler;
 int g_amiga_savestate_docompress = 1;
 
 #ifdef DEBUG_SYNC
@@ -126,15 +128,13 @@ void amiga_floppy_set_writable_images(int writable) {
     g_fs_uae_writable_disk_images = writable;
 }
 
-int amiga_init()
+int amiga_init(void)
 {
     printf("UAE: Initializing core derived from %s\n", UAE_BASE_VERSION);
     write_log("UAE: Initializing core derived from %s\n", UAE_BASE_VERSION);
 
-    // because frame_time_t is sometimes cast to int, we make sure to
-    // start from 0 wo it will work for "a while". Should be fixed
-    // properly
-    g_uae_epoch = fs_get_monotonic_time();
+    uae_register_main_thread();
+    uae_time_init();
 
     /*
 #ifdef DEBUG_SYNC
@@ -151,10 +151,17 @@ int amiga_init()
     filesys_host_init();
 
     romlist_init();
-
+    clipboard_init();
     return 1;
 }
 
+bool amiga_init_jit_compiler(void)
+{
+    write_log("JIT: Enabling JIT compiler\n");
+    g_fs_uae_jit_compiler = true;
+    return true;
+}
+
 void amiga_set_video_format(int format) {
     g_amiga_video_format = format;
     if (format == AMIGA_VIDEO_FORMAT_R5G6B5) {
@@ -278,17 +285,25 @@ int amiga_get_rand_checksum() {
     return uaerand() & 0x00ffffff;
 }
 
-int amiga_get_state_checksum() {
-    int checksum = uae_get_memory_checksum();
+int amiga_get_state_checksum(void)
+{
+    int checksum = uae_get_memory_checksum(NULL, 0);
 #ifdef DEBUG_SYNC
     write_sync_log("memcheck: %08x\n", checksum);
 #endif
     return checksum & 0x00ffffff;
 }
 
-//int amiga_main(int argc, char** argv) {
-void amiga_main() {
+int amiga_get_state_checksum_and_dump(void *data, int size)
+{
+    int checksum = uae_get_memory_checksum(data, size);
+    return checksum & 0x00ffffff;
+}
+
+void amiga_main(void)
+{
     write_log("amiga_main\n");
+    uae_register_emulation_thread();
 
     keyboard_settrans();
 
@@ -311,7 +326,8 @@ void amiga_write_config(const char *path) {
     cfgfile_save(&currprefs, path, 0);
 }
 
-int amiga_enable_serial_port(const char *serial_name) {
+int amiga_enable_serial_port(const char *serial_name)
+{
     write_log("amiga_enable_serial_port\n");
     if (serial_name != NULL && serial_name[0]) {
         write_log("serial port device: %s\n", serial_name);
@@ -319,14 +335,21 @@ int amiga_enable_serial_port(const char *serial_name) {
         strcpy(currprefs.sername, serial_name);
         changed_prefs.use_serial = 1;
         currprefs.use_serial = 1;
+        return 1;
     }
-    /*
-    else {
-        write_log("serial: using dummy serial port\n");
+    return 0;
+}
+
+int amiga_enable_parallel_port(const char *parallel_name)
+{
+    write_log("amiga_enable_parallel_port\n");
+    if (parallel_name != NULL && parallel_name[0]) {
+        write_log("Parallel port device: %s\n", parallel_name);
+        strcpy(changed_prefs.prtname, parallel_name);
+        strcpy(currprefs.prtname, parallel_name);
+        return 1;
     }
-    */
-    //config_changed = 1;
-    return 1;
+    return 0;
 }
 
 void amiga_set_cpu_idle(int idle)
